@@ -6,9 +6,17 @@ our ($ROUND, $PREC);
 # Implemented as:
 #    beta(x,y) = gamma(x)*gamma(y) / gamma(x+y)
 
+state $has_beta = Math::MPFR::MPFR_VERSION_MAJOR() >= 4;
+
 sub __beta__ {
     my ($x, $y) = @_;
 
+    if ($has_beta) {    # available since mpfr-4.0.0
+        my $r = Math::MPFR::Rmpfr_init2($PREC);
+        Math::MPFR::Rmpfr_beta($r, $x, $y, $ROUND);
+        return $r;
+    }
+
     my $t1 = Math::MPFR::Rmpfr_init2($PREC);    # gamma(x+y)
     my $t2 = Math::MPFR::Rmpfr_init2($PREC);    # gamma(y)
 
