@@ -31,7 +31,7 @@ sub __eq__ {
         return (
                 Math::MPFR::Rmpfr_integer_p($x)
                   and (
-                       $y < 0
+                       ($y || return !Math::MPFR::Rmpfr_sgn($x)) < 0
                        ? Math::MPFR::Rmpfr_cmp_si($x, $y)
                        : Math::MPFR::Rmpfr_cmp_ui($x, $y)
                   ) == 0
@@ -62,7 +62,7 @@ sub __eq__ {
         return (
                 Math::GMPq::Rmpq_integer_p($x)
                   and (
-                       $y < 0
+                       ($y || return !Math::GMPq::Rmpq_sgn($x)) < 0
                        ? Math::GMPq::Rmpq_cmp_si($x, $y, 1)
                        : Math::GMPq::Rmpq_cmp_ui($x, $y, 1)
                   ) == 0
@@ -92,7 +92,7 @@ sub __eq__ {
   Math_GMPz__Scalar: {
         return (
                 (
-                 $y < 0
+                 ($y || return !Math::GMPz::Rmpz_sgn($x)) < 0
                  ? Math::GMPz::Rmpz_cmp_si($x, $y)
                  : Math::GMPz::Rmpz_cmp_ui($x, $y)
                 ) == 0
